@@ -17,6 +17,7 @@ import random
 import time
 import tracemalloc
 import psutil
+import anndata
 from sklearn import metrics
 from sklearn.decomposition import PCA
 from sklearn.preprocessing import MinMaxScaler
@@ -535,10 +536,17 @@ def GetStrategy(mat,jac):
 
 ###define main function here
 class KNNIter(object):
-    def __init__(self, rawmat, strategy = None, core = None, sim_rate= None, nPC=None, neigbors=None, nTree=None, label= None, exprate = None):
+    def __init__(self, rawmat, strategy = None, core = None, sim_rate= None, nPC=None, neighbors=None, nTree=None, label= None, exprate = None):
         ##preprocess1, select the peaks that are open in more than 1% cells
+        if isinstance(rawmat, anndata.AnnData):
+            ##判断是否是adata格式输入
+            rawmat_ann = rawmat
+            rawmat = rawmat.X
+            
+        else:
+            rawmat = rawmat
         rawmat_pf = rawmat[:,(rawmat.sum(axis = 0)> 0.01* rawmat.shape[0]).A.squeeze()]
-        print(rawmat_pf.shape)
+#         print(rawmat_pf.shape)
         self.rawmat = rawmat
         self.fmat = rawmat_pf
         
@@ -559,10 +567,10 @@ class KNNIter(object):
             self.nPC = 5
         else:
             self.nPC = nPC
-        if neigbors is None:
-            self.neigbors = 40
+        if neighbors is None:
+            self.neighbors = 40
         else:
-            self.neigbors = neigbors
+            self.neighbors = neighbors
         if nTree is None:
             self.nTree = 30
         else:
@@ -587,7 +595,7 @@ class KNNIter(object):
 #     def IterCall(self):
         
         if mat is None:
-            mat = self.rawmat 
+            mat = self.fmat 
         if isinstance(mat, np.ndarray):
             mat = scipy.sparse.csr_matrix(mat)
         if strategy is None:
@@ -599,7 +607,7 @@ class KNNIter(object):
         if npc is None:
             npc = self.nPC
         if k is None:
-            k = self.neigbors
+            k = self.neighbors
         if n_tree is None:
             n_tree = self.nTree
         if labelmat is None:
@@ -681,5 +689,16 @@ class KNNIter(object):
         proba_final = np.mean(score_pool,axis = 0)
         min_max_scaler = MinMaxScaler( )
         proba_final = min_max_scaler.fit_transform(proba_final.reshape(-1, 1)).squeeze()
-        return label_refer,score_pool,proba_final
+        thresh_final = np.quantile(proba_final,1-exprate,interpolation= 'higher')
+        ####如果是ann格式输入，返回一个ann;如果是numpy输入，返回predicted doublet序号及doublet score
+        try:
+            rawmat_ann
+            pred_results = np.zeros(proba_final.shape[0])
+            pred_results[proba_final > thresh_final] = 1
+            rawmat_ann.obs['PredDBL'] = pred_results
+            rawmat_ann.obs['DBLscore'] = proba_final
+            return rawmat_ann
+        except:
+            return labelmat[proba_final > thresh_final],proba_final
+
         
\ No newline at end of file
